/*
 * Copyright (c) 2004-2014, Willem Cazander
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package	org.x4o.xml.io.sax.ext;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;


/**
 * ContentWriter is ContentHandler for writing sax events.
 * 
 * @author Willem Cazander
 * @version 1.0 Apr 30, 2013
 */
public interface ContentWriter extends ContentHandler,LexicalHandler {
	
	/**
	 * Starts and ends an element in one call.
	 * @param uri	The uri of the element.
	 * @param localName	The localName of the element.
	 * @param name	The name of the element.
	 * @param atts	The attributes of the element.
	 * @throws SAXException	When IOException is thrown.
	 */
	void startElementEnd(String uri, String localName, String name,Attributes atts) throws SAXException;
	
	/**
	 * Writes a String as xml comment.
	 * @param text	The text to have embedded as comment in xml.
	 * @throws SAXException	On error.
	 */
	void comment(String text) throws SAXException;
	
	/**
	 * Writes a whitespace String to the body.
	 * @param text	The String of whitespace to write.
	 * @throws SAXException	On error.
	 */
	void ignorableWhitespace(String text) throws SAXException;
	
	/**
	 * Writes a whitespace character to the body.
	 * @param c	The character to write.
	 * @throws SAXException On error;
	 */
	void ignorableWhitespace(char c) throws SAXException;
	
	/**
	 * Writes a String in the body.
	 * @param text	The text to write.
	 * @throws SAXException	On error.
	 */
	void characters(String text) throws SAXException;
	
	/**
	 * Writes a single character in the body body.
	 * @param c	The character to write.
	 * @throws SAXException	On error.
	 */
	void characters(char c) throws SAXException;
}
